import urllib.parse

from ruia import Request, Spider

from AnimeCrawler.utils import is_url

from .log import Logger


class BaseSpider(Spider):
    '''所有爬虫的基类，必须提供domain与downloader属性

    Args:
        Spider (ruia.Spider): 继承于ruia框架的爬虫

    Raises:
        ValueError: 未定义domain时报错
        ValueError: 未定义downloader时报错

    Returns:
        cls: 为了链式调用ruia.Spider.start()
    '''

    logger = Logger()

    def __init__(
        self,
        *spider_args,
        is_debug=False,
        **spider_kwargs,
    ) -> None:
        super().__init__(*spider_args, **spider_kwargs)
        if not hasattr(self, 'domain'):
            raise ValueError(f'{self.__class__.__name__} 未定义domain属性')
        elif not hasattr(self, 'downloader'):
            raise ValueError(f'{self.__class__.__name__} 未定义downloader下载器')

    @classmethod
    def init(cls, is_debug):
        if is_debug:
            cls.logger.level = 'DEBUG'
        cls.logger.get_logger('Spider')
        cls.start_urls = [
            i if is_url(i) else urllib.parse.urljoin(cls.domain, i)
            for i in cls.start_urls
        ]  # 当url为相对路径时与域名拼接
        return cls

    def urljoin(self, base, url, avoid_collision: bool = False) -> str:
        '''对urllib.parse.urljoin()的包装

        Args:
            base (str): 基础url
            url (str): 要拼接的url
            avoid_collision (bool, optional): 避免冲突，

            详情看：https://docs.python.org/zh-cn/3/library/urllib.parse.html#urllib.parse.urljoin

            Defaults to False.

        Returns:
            str: 拼接后的url
        '''
        if avoid_collision:
            url_parts = urllib.parse.urlsplit(url)
            url = urllib.parse.urlunsplit(url_parts._replace(scheme='', netloc=''))
        return urllib.parse.urljoin(base, url)

    async def follow(self, next_url: str = None, **kwargs) -> Request:
        '''爬取下一个页面

        Args:
            next_url (str, optional): 下一个url的相对路径. Defaults to None.

        Returns:
            ruia.Request: 可被yield
        '''
        return self.request(self.urljoin(self.domain, next_url), **kwargs)

    def get_domain(self, url: str) -> str:
        url_parts = urllib.parse.urlsplit(url)
        return '://'.join(url_parts[:2])  # e.g. https://docs.python.org/

    def get_path(self, url: str) -> str:
        url_parts = urllib.parse.urlsplit(url)
        return url_parts.path


if __name__ == '__main__':
    BaseSpider().get_domain(
        'https://docs.python.org/zh-cn/3/library/urllib.parse.html#urllib.parse.urljoin'
    )



import requests

a = requests.get(
    'https://www.mhyyy.com/search.html?wd=%E4%B8%AD%E4%BA%8C%E7%97%85',
    headers={'User-Agent': 'Mozilla/5.0'},
).text
with open('a.html', 'a+') as fp:
    fp.write(a)

'//div[@class=module-items module-card-items]'



from AnimeCrawler.base_spider import BaseSpider
from AnimeCrawler.cli import main
from AnimeCrawler.log import Logger
from AnimeCrawler.mhyyy import AnimeSpider, Downloader, Searcher
from AnimeCrawler.utils import (
    align,
    base64_decode,
    folder_path,
    get_video_path,
    is_url,
    merge_ts2mp4,
    unescape,
    write,
)

__version__ = 'v0.2.1'



import itertools
from typing import Generator

from ruia import AttrField, Item, TextField

from AnimeCrawler.base_spider import BaseSpider
from AnimeCrawler.utils import align


# from AnimeCrawler.log import get_logger
class SearchItem(Item):
    target_item = TextField(
        xpath_select='//div[@class="module-items module-card-items"]'
    )
    url = AttrField(
        attr='href',
        xpath_select='//div[@class="module-card-item-footer"]/a[@class="play-btn icon-btn"]',
        many=True,
    )
    title = TextField(
        xpath_select='//div[@class="module-card-item-title"]/a[@rel="nofollow"]/strong',
        many=True,
    )
    info = TextField(
        xpath_select='//div[@class="module-card-item-info"]/div[last()]/div["module-card-item-content"]',
        many=True,
    )


class LineItem(Item):
    target_item = TextField(xpath_select='//div[@class="swiper-wrapper"]')
    line_name = TextField(
        xpath_select='//a[@data-hash="slide{1}"]',
        many=True,
    )
    line_url = AttrField(
        attr='href',
        xpath_select='//a[@data-hash="slide{1}"]',
        many=True,
    )


class Searcher(BaseSpider):
    session = None
    downloader = None
    domain = 'https://www.mh620.com'

    @classmethod
    def init(cls, anime_title, is_debug) -> BaseSpider:
        cls.start_urls = [
            cls.urljoin(cls, cls.domain, f'/search.html?wd={anime_title}')
        ]
        return super().init(is_debug)

    async def select_anime(self, animes) -> None:
        answer = input('Which anime do you want to download? >>> ')
        if answer.isdigit():
            answer = int(answer)
            anime = animes[answer - 1][1]
            title, url = anime[0].replace(' ', '_'), self.domain + anime[1]
            html = await self.request(url=url).fetch()
            await self.select_line(title, html)

    async def select_line(self, title, html):
        async for item in LineItem.get_items(html=await html.text()):
            lines = tuple(
                enumerate(
                    zip(item.line_name, item.line_url, itertools.repeat(None)),
                    start=1,
                )
            )
            await self.pretty_print(lines)
        answer = input('Which line do you expect to download? >>> ')
        if answer.isdigit():
            answer = int(answer)
            line_info = lines[answer - 1][1]
            print(
                f'\nDownload Command:\nAnimeCrawler download -t {title} -u {self.domain + line_info[1]} --del_ts'
            )

    async def pretty_print(self, items) -> None:
        partten = "{0}| {1}| {2}"
        print(partten.format('序号', align('标题', 38, 'C'), align('简介', 26, 'C')))
        print('-' * 75)
        for index, (title, _, info) in items:
            info = '无简介' if info is None else info[:12] + '...'
            print(
                partten.format(align(str(index), 4), align(title, 37), align(info, 20))
            )

    async def parse(self, response) -> Generator[SearchItem, None, None]:
        async for item in SearchItem.get_items(html=await response.text()):
            yield item

    async def process_item(self, item: SearchItem) -> None:
        animes = tuple(enumerate(zip(item.title, item.url, item.info), start=1))
        # 美化输出
        await self.pretty_print(animes)
        await self.select_anime(animes)



import abc
import argparse
from collections import namedtuple

from AnimeCrawler.log import Logger
from AnimeCrawler.mhyyy.searcher import Searcher
from AnimeCrawler.mhyyy.spider import AnimeSpider
from AnimeCrawler.utils import is_url


class BaseCommand:
    '''命令的基类

    子类必须实现 subcommand_add_arguments, handle, catch_error 方法
    '''

    logger = Logger()

    @abc.abstractmethod
    def subcommand_add_arguments(self, parser) -> None:
        ...

    @abc.abstractmethod
    def handle(self, args) -> None:
        ...

    @property
    def base_error(self):
        Errors = namedtuple("Errors", ('error_code', 'error_reason', 'output'))
        return Errors


class DownloadCommand(BaseCommand):
    def subcommand_add_arguments(self, parser: argparse._SubParsersAction):
        parser.add_argument(
            "-u",
            "--url",
            help="动漫第一集的url",
        )
        parser.add_argument(
            "-t",
            "--title",
            metavar='Title',
            help="动漫名称",
        )
        parser.add_argument(
            "--del_ts", dest='can_del_ts', help="删除ts文件", action='store_true'
        )
        parser.add_argument(
            '--debug', dest='is_debug', help='进入debug模式', action='store_true'
        )

    def handle(self, args) -> None:
        if error := self.catch_error(args):
            raise ValueError(f'{error.output}')
        AnimeSpider.init(args.title, args.url, args.can_del_ts).start()

    def catch_error(self, parse):
        if not parse.title:
            return self.base_error('402', 'null_title', '标题为空')
        elif not is_url(parse.url or ''):
            return self.base_error('403', 'is_not_url', f'{parse.url} 不为合法的url')


class SearchCommand(BaseCommand):
    def subcommand_add_arguments(self, parser) -> None:
        parser.add_argument(
            "-t",
            "--title",
            metavar='Title',
            help="动漫名称",
        )
        parser.add_argument(
            '--debug', dest='is_debug', help='进入debug模式', action='store_true'
        )

    def handle(self, args) -> None:
        if error := self.catch_error(args):
            raise ValueError(f'{error.output}')
        Searcher.init(args.title, is_debug=args.is_debug).start()

    def catch_error(self, parse):
        if not parse.title:
            return self.base_error('402', 'null_title', '标题为空')


def main():
    subcommands = {
        'download': (DownloadCommand, '下载动漫'),
        'search': (SearchCommand, '搜索动漫'),
    }
    parser = argparse.ArgumentParser(
        prog='AnimeCrawler',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description='* AnimeCrawler v0.2.0 - 一个可免费下载动漫的爬虫\n* Repo: https://github.com/Senvlin/AnimeCrawler',
        epilog='Had Issues? Go To -> https://github.com/Senvlin/AnimeCrawler/issues',
    )
    subparsers: argparse._SubParsersAction = parser.add_subparsers(prog='AnimeCrawler')
    for name, profile in subcommands.items():
        cmd: BaseCommand = profile[0]()
        subparser = subparsers.add_parser(name, help=profile[1])
        subparser.set_defaults(handle=cmd.handle)
        cmd.subcommand_add_arguments(subparser)
    args = parser.parse_args()
    if hasattr(args, 'handle'):
        args.handle(args)



import logging

_str_to_level = {
    'DEBUG': logging.DEBUG,
    'INFO': logging.INFO,
    'WARNING': logging.WARNING,
    'ERROR': logging.ERROR,
}


class Logger:
    _level = 'ERROR'

    @property
    def level(self, level):
        return level

    @level.setter
    def level(self, level):
        if _str_to_level.get(level, None):
            print(level)
            self._level = level

    def get_logger(self, name='') -> logging.Logger:
        logging.basicConfig(
            level=self._level,
            format='[\033[0;36;40m%(asctime)s.%(msecs)03d\033[0m] %(name)-7s | %(levelname)-7s | %(filename)s - %(lineno)d: %(message)s',
            datefmt='%Y-%d-%m %I:%M:%S',
        )
        logger = logging.getLogger(name)
        return logger
